@@ -152,6 +152,9 @@ void player::reset_playstate()
     _pause_request = false;
     _seek_request = 0;
     _set_pos_request = -1.0f;
+    _video_frame = video_frame();
+    _current_subtitle_box = subtitle_box();
+    _next_subtitle_box = subtitle_box();
 }
 
 void player::stop_playback()
@@ -174,11 +177,21 @@ video_output *player::create_video_output()
     return new video_output_qt(_benchmark);
 }
 
+void player::destroy_video_output(video_output *vo)
+{
+    delete vo;
+}
+
 audio_output *player::create_audio_output()
 {
     return new audio_output();
 }
 
+void player::destroy_audio_output(audio_output *ao)
+{
+    delete ao;
+}
+
 void player::make_master()
 {
     controller::set_global_player(this);
@@ -685,6 +698,10 @@ bool player::run_step()
     }
     if (prep_frame)
     {
+        if (_current_subtitle_box.is_valid() && _video_output)
+        {
+            _master_time_start += _video_output->wait_for_subtitle_renderer();
+        }
         _video_output->prepare_next_frame(_video_frame, _current_subtitle_box);
     }
     else if (drop_frame)
@@ -716,13 +733,13 @@ void player::close()
     if (_audio_output)
     {
         try { _audio_output->deinit(); } catch (...) {}
-        delete _audio_output;
+        destroy_audio_output(_audio_output);
         _audio_output = NULL;
     }
     if (_video_output)
     {
         try { _video_output->deinit(); } catch (...) {}
-        delete _video_output;
+        destroy_video_output(_video_output);
         _video_output = NULL;
     }
     if (_media_input)
