@@ -4,6 +4,7 @@
  * Copyright (C) 2011
  * Martin Lambers <marlam@marlam.de>
  * Joe <cuchac@email.cz>
+ * Frédéric Devernay <Frederic.Devernay@inrialpes.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +33,9 @@
 #if (defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__
 # include <windows.h>
 #endif
+#if defined __APPLE__
+# include <unistd.h>
+#endif
 
 extern "C"
 {
@@ -41,6 +45,7 @@ extern "C"
 #include "gettext.h"
 #define _(string) gettext(string)
 
+#include "dbg.h"
 #include "exc.h"
 #include "str.h"
 #include "blob.h"
@@ -50,6 +55,17 @@ extern "C"
 #include "subtitle_renderer.h"
 
 
+subtitle_renderer_initializer::subtitle_renderer_initializer(subtitle_renderer &renderer) :
+    _subtitle_renderer(renderer)
+{
+}
+
+void subtitle_renderer_initializer::run()
+{
+    _subtitle_renderer.init();
+}
+
+
 /* Rendering subtitles with LibASS is not thread-safe.
  *
  * We have multiple concurrent subtitle rendering threads if we are running from Equalizer
@@ -66,16 +82,28 @@ extern "C"
 static mutex global_libass_mutex;
 
 subtitle_renderer::subtitle_renderer() :
-    _ass_initialized(false),
+    _initializer(*this),
+    _initialized(false),
     _fontconfig_conffile(NULL),
     _ass_library(NULL),
     _ass_renderer(NULL),
     _ass_track(NULL)
 {
+    _initializer.start();
 }
 
 subtitle_renderer::~subtitle_renderer()
 {
+    if (_initializer.is_running())
+    {
+        try
+        {
+            _initializer.cancel();
+        }
+        catch (...)
+        {
+        }
+    }
     if (_ass_track)
     {
         ass_free_track(_ass_track);
@@ -94,6 +122,149 @@ subtitle_renderer::~subtitle_renderer()
     }
 }
 
+static void libass_msg_callback(int level, const char *fmt, va_list args, void *)
+{
+    msg::level_t msg_levels[8] = { msg::ERR, msg::ERR, msg::WRN, msg::WRN, msg::WRN, msg::INF, msg::DBG, msg::DBG };
+    msg::level_t l = (level < 0 || level > 7 ? msg::ERR : msg_levels[level]);
+    std::string s = str::vasprintf(fmt, args);
+    msg::msg(l, std::string("LibASS: ") + s);
+}
+
+const char *subtitle_renderer::get_fontconfig_conffile()
+{
+#if ((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__) || defined __APPLE__
+    /* Fontconfig annoyingly requires a configuration file, but there is no
+     * default location for it on Windows or Mac OS X. So we just create a temporary one.
+     * Note that if something goes wrong and we return NULL here, the application will
+     * likely crash when trying to render a subtitle. */
+    FILE *f;
+# if ((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
+    /* Note that due to lack of mkstemp() on Windows, this code has race conditions. */
+    char tmppathname[MAX_PATH];
+    static char tmpfilename[MAX_PATH];
+    DWORD ret;
+    ret = GetTempPath(MAX_PATH, tmppathname);
+    if (ret > MAX_PATH || ret == 0)
+        return NULL;
+    if (GetTempFileName(tmppathname, "fonts_conf", 0, tmpfilename) == 0)
+        return NULL;
+    if (!(f = fopen(tmpfilename, "w")))
+        return NULL;
+# else /* __APPLE__ */
+    static char tmpfilename[] = "/tmp/fontsXXXXXX.conf";
+    int d = mkstemps(tmpfilename, 5);
+    if (d == -1)
+        return NULL;
+    if (!(f = fdopen(d, "w")))
+        return NULL;
+# endif
+    fprintf(f,
+            "<?xml version=\"1.0\"?>\n"
+            "<!DOCTYPE fontconfig SYSTEM \"fonts.dtd\">\n"
+            "<fontconfig>\n"
+# if ((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
+            "<dir>WINDOWSFONTDIR</dir>\n"
+            "<dir>~/.fonts</dir>\n"
+            "<cachedir>WINDOWSTEMPDIR_FONTCONFIG_CACHE</cachedir>\n"
+            "<cachedir>~/.fontconfig</cachedir>\n"
+# else /* __APPLE__ */
+            "<dir>/usr/share/fonts</dir>\n"
+            "<dir>/usr/X11/lib/X11/fonts</dir>\n"
+            "<dir>/usr/X11/share/fonts</dir>\n"
+            "<dir>/opt/X11/share/fonts</dir>\n"
+            "<dir>/Library/Fonts</dir>\n"
+            "<dir>/Network/Library/Fonts</dir>\n"
+            "<dir>/System/Library/Fonts</dir>\n"
+            "<dir>~/Library/Application Support/Bino/fonts</dir>\n"
+            "<cachedir>/var/cache/fontconfig</cachedir>\n"
+            "<cachedir>/usr/X11/var/cache/fontconfig</cachedir>\n"
+            "<cachedir>/opt/X11/var/cache/fontconfig</cachedir>\n"
+            "<cachedir>~/Library/Application Support/Bino/cache/fonts</cachedir>\n"
+            "<cachedir>~/.fontconfig</cachedir>\n"
+# endif
+            "<config>\n"
+            "<blank>\n"
+            "<int>0x0020</int> <int>0x00A0</int> <int>0x00AD</int> <int>0x034F</int> <int>0x0600</int>\n"
+            "<int>0x0601</int> <int>0x0602</int> <int>0x0603</int> <int>0x06DD</int> <int>0x070F</int>\n"
+            "<int>0x115F</int> <int>0x1160</int> <int>0x1680</int> <int>0x17B4</int> <int>0x17B5</int>\n"
+            "<int>0x180E</int> <int>0x2000</int> <int>0x2001</int> <int>0x2002</int> <int>0x2003</int>\n"
+            "<int>0x2004</int> <int>0x2005</int> <int>0x2006</int> <int>0x2007</int> <int>0x2008</int>\n"
+            "<int>0x2009</int> <int>0x200A</int> <int>0x200B</int> <int>0x200C</int> <int>0x200D</int>\n"
+            "<int>0x200E</int> <int>0x200F</int> <int>0x2028</int> <int>0x2029</int> <int>0x202A</int>\n"
+            "<int>0x202B</int> <int>0x202C</int> <int>0x202D</int> <int>0x202E</int> <int>0x202F</int>\n"
+            "<int>0x205F</int> <int>0x2060</int> <int>0x2061</int> <int>0x2062</int> <int>0x2063</int>\n"
+            "<int>0x206A</int> <int>0x206B</int> <int>0x206C</int> <int>0x206D</int> <int>0x206E</int>\n"
+            "<int>0x206F</int> <int>0x2800</int> <int>0x3000</int> <int>0x3164</int> <int>0xFEFF</int>\n"
+            "<int>0xFFA0</int> <int>0xFFF9</int> <int>0xFFFA</int> <int>0xFFFB</int>\n"
+            "</blank>\n"
+            "<rescan><int>30</int></rescan>\n"
+            "</config>\n"
+            "</fontconfig>\n");
+    fclose(f);
+    return tmpfilename;
+#else
+    /* Systems other than Windows and Mac OS are expected to have a default
+     * fontconfig configuration file so that we can simply return NULL. */
+    return NULL;
+#endif
+}
+
+void subtitle_renderer::init()
+{
+    if (!_initialized)
+    {
+        try
+        {
+            global_libass_mutex.lock();
+
+            ASS_Library *ass_library = ass_library_init();
+            if (!ass_library)
+            {
+                throw exc(_("Cannot initialize LibASS."));
+            }
+            ass_set_message_cb(ass_library, libass_msg_callback, NULL);
+            ass_set_extract_fonts(ass_library, 1);
+            _ass_library = ass_library;
+
+            ASS_Renderer *ass_renderer = ass_renderer_init(_ass_library);
+            if (!ass_renderer)
+            {
+                throw exc(_("Cannot initialize LibASS renderer."));
+            }
+            ass_set_hinting(ass_renderer, ASS_HINTING_NATIVE);
+            _fontconfig_conffile = get_fontconfig_conffile();
+            ass_set_fonts(ass_renderer, NULL, "sans-serif", 1, _fontconfig_conffile, 1);
+            _ass_renderer = ass_renderer;
+
+            _initialized = true;
+            global_libass_mutex.unlock();
+        }
+        catch (...)
+        {
+            // Either we threw this exception ourselves, or the thread was cancelled.
+            // Note that if the thread was cancelled, we likely leak ressources.
+            // But at least we avoid to deconstruct potentially inconsistent ASS_Library
+            // and ASS_Renderer objects.
+            global_libass_mutex.unlock();
+            throw;
+        }
+    }
+}
+
+bool subtitle_renderer::is_initialized()
+{
+    if (_initializer.is_running())
+    {
+        return false;
+    }
+    else
+    {
+        // rethrow a possible exception if something went wrong
+        _initializer.finish();
+        return _initialized;
+    }
+}
+
 bool subtitle_renderer::render_to_display_size(const subtitle_box &box) const
 {
     return (box.format != subtitle_box::image);
@@ -104,12 +275,12 @@ void subtitle_renderer::prerender(const subtitle_box &box, int64_t timestamp,
         int width, int height, float pixel_aspect_ratio,
         int &bb_x, int &bb_y, int &bb_w, int &bb_h)
 {
+    assert(_initialized);
     _fmt = box.format;
     switch (_fmt)
     {
     case subtitle_box::text:
     case subtitle_box::ass:
-        init_ass();
         prerender_ass(box, timestamp, params, width, height, pixel_aspect_ratio);
         break;
     case subtitle_box::image:
@@ -136,120 +307,6 @@ void subtitle_renderer::render(uint32_t *bgra32_buffer)
     }
 }
 
-const char *subtitle_renderer::get_fontconfig_conffile()
-{
-#if (defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__
-    /* Fontconfig annoyingly requires a configuration file, but there is no
-     * default location for it on Windows. So we just create a temporary one.
-     * Note that due to lack of mkstemp() on Windows, this code has race
-     * conditions. Note also that if something goes wrong and we return NULL
-     * here, the application will likely crash when we try to render a subtitle. */
-    char tmppathname[MAX_PATH];
-    static char tmpfilename[MAX_PATH];
-    FILE *f;
-    DWORD ret;
-    ret = GetTempPath(MAX_PATH, tmppathname);
-    if (ret > MAX_PATH || ret == 0)
-        return NULL;
-    if (GetTempFileName(tmppathname, "fonts_conf", 0, tmpfilename) == 0)
-        return NULL;
-    if (!(f = fopen(tmpfilename, "w")))
-        return NULL;
-    fprintf(f,
-            "<?xml version=\"1.0\"?>\n"
-            "<!DOCTYPE fontconfig SYSTEM \"fonts.dtd\">\n"
-            "<fontconfig>\n"
-            "<dir>WINDOWSFONTDIR</dir>\n"
-            "<dir>~/.fonts</dir>\n"
-            "<cachedir>WINDOWSTEMPDIR_FONTCONFIG_CACHE</cachedir>\n"
-            "<cachedir>~/.fontconfig</cachedir>\n"
-            "<config>\n"
-            "<blank>\n"
-            "<int>0x0020</int> <int>0x00A0</int> <int>0x00AD</int> <int>0x034F</int> <int>0x0600</int>\n"
-            "<int>0x0601</int> <int>0x0602</int> <int>0x0603</int> <int>0x06DD</int> <int>0x070F</int>\n"
-            "<int>0x115F</int> <int>0x1160</int> <int>0x1680</int> <int>0x17B4</int> <int>0x17B5</int>\n"
-            "<int>0x180E</int> <int>0x2000</int> <int>0x2001</int> <int>0x2002</int> <int>0x2003</int>\n"
-            "<int>0x2004</int> <int>0x2005</int> <int>0x2006</int> <int>0x2007</int> <int>0x2008</int>\n"
-            "<int>0x2009</int> <int>0x200A</int> <int>0x200B</int> <int>0x200C</int> <int>0x200D</int>\n"
-            "<int>0x200E</int> <int>0x200F</int> <int>0x2028</int> <int>0x2029</int> <int>0x202A</int>\n"
-            "<int>0x202B</int> <int>0x202C</int> <int>0x202D</int> <int>0x202E</int> <int>0x202F</int>\n"
-            "<int>0x205F</int> <int>0x2060</int> <int>0x2061</int> <int>0x2062</int> <int>0x2063</int>\n"
-            "<int>0x206A</int> <int>0x206B</int> <int>0x206C</int> <int>0x206D</int> <int>0x206E</int>\n"
-            "<int>0x206F</int> <int>0x2800</int> <int>0x3000</int> <int>0x3164</int> <int>0xFEFF</int>\n"
-            "<int>0xFFA0</int> <int>0xFFF9</int> <int>0xFFFA</int> <int>0xFFFB</int>\n"
-            "</blank>\n"
-            "<rescan><int>30</int></rescan>\n"
-            "</config>\n"
-            "</fontconfig>\n");
-    fclose(f);
-    return tmpfilename;
-#else
-    return NULL;
-#endif
-}
-
-static void libass_msg_callback(int level, const char *fmt, va_list args, void *)
-{
-    msg::level_t l;
-    switch (level)
-    {
-    default:
-    case 0:
-        l = msg::ERR;
-        break;
-    case 1:
-        l = msg::ERR;
-        break;
-    case 2:
-        l = msg::WRN;
-        break;
-    case 3:
-        l = msg::WRN;
-        break;
-    case 4:
-        l = msg::WRN;
-        break;
-    case 5:
-        l = msg::INF;
-        break;
-    case 6:
-        l = msg::DBG;
-        break;
-    case 7:
-        l = msg::DBG;
-        break;
-    }
-    std::string s = str::vasprintf(fmt, args);
-    msg::msg(l, std::string("LibASS: ") + s);
-}
-
-void subtitle_renderer::init_ass()
-{
-    if (!_ass_initialized)
-    {
-        global_libass_mutex.lock();
-        _ass_library = ass_library_init();
-        if (!_ass_library)
-        {
-            global_libass_mutex.unlock();
-            throw exc(_("Cannot initialize LibASS."));
-        }
-        ass_set_message_cb(_ass_library, libass_msg_callback, NULL);
-        ass_set_extract_fonts(_ass_library, 1);
-        _ass_renderer = ass_renderer_init(_ass_library);
-        if (!_ass_renderer)
-        {
-            global_libass_mutex.unlock();
-            throw exc(_("Cannot initialize LibASS renderer."));
-        }
-        ass_set_hinting(_ass_renderer, ASS_HINTING_NATIVE);
-        _fontconfig_conffile = get_fontconfig_conffile();
-        ass_set_fonts(_ass_renderer, NULL, "sans-serif", 1, _fontconfig_conffile, 1);
-        _ass_initialized = true;
-        global_libass_mutex.unlock();
-    }
-}
-
 void subtitle_renderer::blend_ass_image(const ASS_Image *img, uint32_t *buf)
 {
     const unsigned int R = (img->color >> 24u) & 0xffu;
