@@ -83,7 +83,6 @@ protected:
 class video_output_qt : public video_output
 {
 private:
-    bool _qt_app_owner;
     video_container_widget *_container_widget;
     bool _container_is_external;
     video_output_qt_widget *_widget;
@@ -123,6 +122,7 @@ public:
     void grab_focus();
 
     virtual void init();
+    virtual int64_t wait_for_subtitle_renderer();
     virtual void deinit();
 
     virtual bool supports_stereo() const;
